@@ -76,6 +76,12 @@ class Cache:
         key_name = tile_key(layer, coord, format)
         self.bucket.delete_key(key_name+'-lock')
         
+    def remove(self, layer, coord, format):
+        """ Remove a cached tile.
+        """
+        key_name = tile_key(layer, coord, format)
+        self.bucket.delete_key(key_name)
+        
     def read(self, layer, coord, format):
         """ Read a cached tile.
         """
