@@ -8,7 +8,7 @@ designers and cartographers.
 
 Documentation available at http://tilestache.org/doc/
 """
-__version__ = '1.27.1'
+__version__ = '1.31.0'
 
 import re
 
@@ -24,6 +24,7 @@ from urlparse import urljoin, urlparse
 from urllib import urlopen
 from os import getcwd
 from time import time
+import logging
 
 try:
     from json import load as json_load
@@ -32,14 +33,11 @@ except ImportError:
 
 from ModestMaps.Core import Coordinate
 
-import Core
-import Config
-
 # dictionary of configuration objects for requestLayer().
 _previous_configs = {}
 
-# dictionary of tiles seen recently in this process, when ignore_cached is on.
-_recent_tiles = {}
+import Core
+import Config
 
 # regular expression for PATH_INFO
 _pathinfo_pat = re.compile(r'^/?(?P<l>\w.+)/(?P<z>\d+)/(?P<x>-?\d+)/(?P<y>-?\d+)\.(?P<e>\w+)$')
@@ -57,26 +55,20 @@ def getTile(layer, coord, extension, ignore_cached=False):
         This is the main entry point, after site configuration has been loaded
         and individual tiles need to be rendered.
     """
+    start_time = time()
+    
     mimetype, format = layer.getTypeByExtension(extension)
     cache = layer.config.cache
 
-    # key in _recent_tiles
-    _tile = (layer, coord, extension)
-    
     if not ignore_cached:
         # Start by checking for a tile in the cache.
         body = cache.read(layer, coord, format)
-
-    elif _tile in _recent_tiles:
-        # Then look in the bag of recent tiles.
-        body, use_by = _recent_tiles[_tile]
-        if time() > use_by:
-            del _recent_tiles[_tile]
-            body = None
+        tile_from = 'cache'
 
     else:
-        # Bypass the cache
-        body = None
+        # Then look in the bag of recent tiles.
+        body = Core._getRecentTile(layer, coord, format)
+        tile_from = 'recent tiles'
     
     # If no tile was found, dig deeper
     if body is None:
@@ -94,6 +86,7 @@ def getTile(layer, coord, extension, ignore_cached=False):
                 # There's a chance that some other process has
                 # written the tile while the lock was being acquired.
                 body = cache.read(layer, coord, format)
+                tile_from = 'cache after all'
     
             if body is None:
                 # No one else wrote the tile, do it here.
@@ -122,13 +115,15 @@ def getTile(layer, coord, extension, ignore_cached=False):
                 if save:
                     cache.save(body, layer, coord, format)
 
+                tile_from = 'layer.render()'
+
         finally:
             if lockCoord:
                 # Always clean up a lock when it's no longer being used.
                 cache.unlock(layer, lockCoord, format)
     
-    if ignore_cached:
-        _recent_tiles[_tile] = body, time() + 300
+    Core._addRecentTile(layer, coord, format, body)
+    logging.info('TileStache.getTile() %s/%d/%d/%d.%s via %s in %.3f', layer.name(), coord.zoom, coord.column, coord.row, extension, tile_from, time() - start_time)
     
     return mimetype, body
 
@@ -193,6 +188,17 @@ def splitPathInfo(pathinfo):
 
     return layer, coord, extension
 
+def mergePathInfo(layer, coord, extension):
+    """ Converts layer name, coordinate and extension back to a PATH_INFO string.
+    
+        See also splitPathInfo().
+    """
+    z = coord.zoom
+    x = coord.column
+    y = coord.row
+    
+    return '/%(layer)s/%(z)d/%(x)d/%(y)d.%(extension)s' % locals()
+
 def requestLayer(config, path_info):
     """ Return a Layer.
     
@@ -272,6 +278,11 @@ def requestHandler(config_hint, path_info, query_string):
         elif extension == 'html' and coord is None:
             mimetype, content = getPreview(layer)
 
+        elif extension.lower() in layer.redirects:
+            other_extension = layer.redirects[extension.lower()]
+            other_path_info = mergePathInfo(layer.name(), coord, other_extension)
+            raise Core.TheTileIsInAnotherCastle(other_path_info)
+        
         else:
             mimetype, content = getTile(layer, coord, extension)
     
@@ -305,7 +316,21 @@ def cgiHandler(environ, config='./tilestache.cfg', debug=False):
     path_info = environ.get('PATH_INFO', None)
     query_string = environ.get('QUERY_STRING', None)
     
-    mimetype, content = requestHandler(config, path_info, query_string)
+    try:
+        mimetype, content = requestHandler(config, path_info, query_string)
+    
+    except Core.TheTileIsInAnotherCastle, e:
+        other_uri = environ['SCRIPT_NAME'] + e.path_info
+        
+        if query_string:
+            other_uri += '?' + query_string
+
+        print >> stdout, 'Status: 302 Found'
+        print >> stdout, 'Location:', other_uri
+        print >> stdout, 'Content-Type: text/plain\n'
+        print >> stdout, 'You are being redirected to', other_uri
+        return
+    
     layer = requestLayer(config, path_info)
     
     if layer.allowed_origin:
@@ -375,7 +400,18 @@ class WSGITileServer:
         if layer and layer not in self.config.layers:
             return self._response(start_response, '404 Not Found')
 
-        mimetype, content = requestHandler(self.config, environ['PATH_INFO'], environ['QUERY_STRING'])
+        try:
+            mimetype, content = requestHandler(self.config, environ['PATH_INFO'], environ['QUERY_STRING'])
+        
+        except Core.TheTileIsInAnotherCastle, e:
+            other_uri = environ['SCRIPT_NAME'] + e.path_info
+            
+            if environ['QUERY_STRING']:
+                other_uri += '?' + environ['QUERY_STRING']
+    
+            start_response('302 Found', [('Location', other_uri), ('Content-Type', 'text/plain')])
+            return ['You are being redirected to %s\n' % other_uri]
+        
         request_layer = requestLayer(self.config, environ['PATH_INFO'])
         allowed_origin = request_layer.allowed_origin
         max_cache_age = request_layer.max_cache_age
