@@ -196,6 +196,8 @@ if __name__ == '__main__':
         
         # override parts of the config and layer if needed
         
+        extension = options.extension
+        
         if options.outputdirectory and options.mbtiles_output:
             cache1 = Disk(options.outputdirectory, dirs='portable', gzip=[])
             cache2 = MBTiles.Cache(options.mbtiles_output, extension, options.layer)
@@ -207,8 +209,6 @@ if __name__ == '__main__':
         elif options.mbtiles_output:
             config.cache = MBTiles.Cache(options.mbtiles_output, extension, options.layer)
         
-        extension = options.extension
-        
         if options.mbtiles_input:
             layer.provider = MBTiles.Provider(layer, options.mbtiles_input)
             n, t, v, d, format, b = MBTiles.tileset_info(options.mbtiles_input)
@@ -249,7 +249,7 @@ if __name__ == '__main__':
     else:
         coordinates = generateCoordinates(ul, lr, zooms, padding)
     
-    coordinates = list(coordinates)[:10]
+    coordinates = list(coordinates)
     
     for (offset, count, coord) in coordinates:
         path = '%s/%d/%d/%d.%s' % (layer.name(), coord.zoom, coord.column, coord.row, extension)
